//
//  CIAttributeLoader.h
//  VDMX
//
//  Created by bagheera on 7/15/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>
#import "VVSnapshotProtocol.h"

#import "VVButton.h"
#import "VVSliderControl.h"
#import "VVLogSliderControl.h"
#import "CIPositionElement.h"
#import "CIRectangleElement.h"
#import "QCTapElement.h"
#import "CIColorElement.h"
#import "VVTextUI.h"
#import "CIStringElement.h"
#import "CIPopUpElement.h"


@interface CIAttributeLoader : NSView <VVSnapshotProtocol> {
	VVScrollView			*parentScrollView;
	NSMutableArray			*elementArray;
	NSMutableArray			*tapArray;
	
	CIFilter				*filter;
	
	BOOL					presetsAreLoading;
	
	pthread_mutex_t			cacheLockA;
	pthread_mutex_t			cacheLockB;
	NSMutableDictionary		*cachedElementTriggersA;
	NSMutableDictionary		*cachedElementTriggersB;
	
	BOOL					deleted;
}

- (void) prepareToBeDeleted;

- (BOOL) hasAttributes;

- (NSMutableDictionary *) createSnapshot;
- (void) loadSnapshot:(NSDictionary *)snap;
- (void) deprecatedLoadSnapshot:(NSDictionary *)snap;

- (void) startPresetRestoreNotification:(NSNotification *)note;
- (void) stopPresetRestoreNotification:(NSNotification *)note;

- (void) loadAttributesForFilter:(CIFilter *)newFilter;
- (void) generateGenericInterfaceForFilter;
- (void) resizeAndReposition;
- (void) removeAllExistingElements;
- (void) setCanvasSize:(NSSize)n;

- (VVButton *) createButtonElementForKey:(NSString *)key;
- (VVSliderControl *) createSliderElementForKey:(NSString *)key;
- (CIColorElement *) createColorElementForKey:(NSString *)key;
- (CIPositionElement *) createPositionElementForKey:(NSString *)key;
- (CIRectangleElement *) createRectangleElementForKey:(NSString *)key;
- (CITapElement *) createTapElementForKey:(NSString *)key;

- (void) VVSliderInterestingPointsChanged:(id)sender;
- (void) VVSliderSyncFlagChanged:(id)sender;
- (void) VVSliderBehavFlagChanged:(id)sender;
- (void) VVSliderMaxEnvChanged:(id)sender;
- (void) VVSliderMinEnvChanged:(id)sender;
- (void) VVButtonSyncFlagChanged:(id)sender;
- (void) VVButtonThreshValChanged:(id)sender;
- (void) VVColorElementChanged:(id)sender;
- (void) VVPositionElementChanged:(id)sender;
- (void) VVTextUIChanged:(id)sender;

- (void) uiElementUsed:(id)sender;
- (void) dumpToFilter;
- (void) processUIElement:(id)sender;
//- (void) popCurrentUIStateToFilter:(CIFilter *)newFilter;

- (void) setFilter:(id)newVal;
- (NSMutableArray *) tapArray;
- (void) setParentScrollView:(id)n;


@end
