//
//  QCAttributeLoader.h
//  SourcePluginFormat
//
//  Created by bagheera on 3/31/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>
#import "VVSnapshotProtocol.h"
#import "VVQCComposition.h"
#import "VVFontUtils.h"
#import "VVButton.h"
#import "VVSliderControl.h"
#import "VVLogSliderControl.h"
#import "CIPositionElement.h"
#import "QCTapElement.h"
#import "CIColorElement.h"
#import "VVTextUI.h"
#import "CIStringElement.h"
#import "CIPopUpElement.h"
#import "CIRectangleElement.h"
#import "VVQCScrollView.h"




@interface QCAttributeLoader : NSView <VVSnapshotProtocol> {
	VVQCScrollView			*parentScrollView;
	NSMutableArray			*elementArray;		//	array of the interface elements
	NSMutableArray			*tabViewArray;		//	array of VVNSTabView objects in my .nib (if there's a .nib)
	NSMutableArray			*tapArray;			//	array of QCTapElements
	NSMutableArray			*bpmInputsArray;		//	array of strings; each string is a key which needs to receive the BPM val
	NSMutableArray			*measurePositionInputsArray;		//	array of strings; each string is a key which needs to receive the measure position val
	BOOL					subscribedToMeasurePosition;
	
	id						defaultFileInput;
	
	id						composition;		//	the composition whose UI i'm displaying- renderer doesn't exist until render thread makes it!
	id						renderer;			//	the QCRenderer whose UI i'm displaying
	CGColorSpaceRef			colorSpace;
	NSMutableDictionary		*currentSnapshotDict;		//	this is the snapshot dict that I'll auto-update when UI objects change
	BOOL					liveUpdating;
	
	id						masterClock;
	id						dataSourceController;
	
	BOOL					presetsAreLoading;
	
	pthread_mutex_t			cacheLockA;
	pthread_mutex_t			cacheLockB;
	NSMutableDictionary		*cachedElementTriggersA;	//	stores ptrs to the UI elements that have been used while i was locked for rendering
	NSMutableDictionary		*cachedElementTriggersB;
	
	BOOL					deleted;
	
	NSView					*topNibView;
	NSSize					topNibViewNativeSize;
}


- (void) prepareToBeDeleted;

- (BOOL) hasAttributes;

- (NSMutableDictionary *) createSnapshot;
- (void) loadSnapshot:(NSDictionary *)snap;
- (void) deprecatedLoadSnapshot:(NSDictionary *)snap;

- (void) startPresetRestoreNotification:(NSNotification *)note;
- (void) stopPresetRestoreNotification:(NSNotification *)note;

- (void) loadAttributesForComposition:(id)newComp;
- (void) loadAttributesForComposition:(id)newComp withNibAtPath:(NSString *)nibPath;

- (void) generateInterfaceForComposition;
- (id) recursivelySearchView:(NSView *)v forElementWithKey:(NSString *)k;
- (void) recursivelyAddTabViewsInView:(NSView *)v toArray:(NSMutableArray *)a;
- (id) findTabViewWithKey:(NSString *)k;
- (id) findElementWithKey:(NSString *)k;
- (void) resizeAndReposition;
- (void) removeAllExistingElements;

- (VVButton *) createButtonElementForKey:(NSString *)key;
- (VVSliderControl *) createSliderElementForKey:(NSString *)key;
- (VVLogSliderControl *) createLogSliderElementForKey:(NSString *)key;
- (CIColorElement *) createColorElementForKey:(NSString *)key;
- (CIPositionElement *) createPositionElementForKey:(NSString *)key;
- (QCTapElement *) createTapElementForKey:(NSString *)key;
- (CIStringElement *) createStringElementForKey:(NSString *)key;
- (CIPopUpElement *) createPopUpElementForKey:(NSString *)key;
- (CIRectangleElement *) createRectangleElementForKey:(NSString *)key;

- (void) populateIndexMenu:(CIPopUpElement *)popUpElement;
- (void) populateFontMenu:(CIPopUpElement *)popUpElement;

//	I'm the delegate for a bunch of sliders and buttons..
- (void) VVSliderInterestingPointsChanged:(id)sender;
- (void) VVSliderSyncFlagChanged:(id)sender;
- (void) VVSliderBehavFlagChanged:(id)sender;
- (void) VVSliderMaxEnvChanged:(id)sender;
- (void) VVSliderMinEnvChanged:(id)sender;
- (void) VVSliderUIStateChanged:(id)sender;
- (void) VVButtonSyncFlagChanged:(id)sender;
- (void) VVButtonThreshValChanged:(id)sender;
- (void) VVButtonUIStateChanged:(id)sender;
- (void) VVColorElementChanged:(id)sender;
- (void) VVPositionElementChanged:(id)sender;
- (void) VVTextUIChanged:(id)sender;
- (void) tabViewElementChanged:(id)sender;

- (void) paramSnapUpdated:(id)sender;
- (void) paramValueUpdated:(id)sender;

- (void) uiElementUsed:(id)sender;
- (void) dumpToRenderer:(QCRenderer *)n;
- (void) dumpToRenderer:(QCRenderer *)n withFallbackImage:(CIImage *)i;
- (void) dumpToRenderer:(QCRenderer *)n withFallbackTexture:(CVOpenGLTextureRef)t;
- (void) dumpMeasurePositions;
- (void) dumpTapsWithFallbackImage:(CIImage *)i;
- (void) dumpTapsWithFallbackTexture:(CVOpenGLTextureRef)t;
- (void) processUIElement:(id)sender;

- (void) masterClockChangedNotification:(NSNotification *)note;

- (void) setDataSourceController:(id)n;
- (void) setComposition:(id)newComp;
- (NSMutableArray *) tapArray;
- (void) setMasterClock:(id)n;
- (id) defaultFileInput;
- (NSMutableDictionary *) currentSnapshotDict;
- (void) setCurrentSnapshotDict:(NSMutableDictionary *) writeToMe;
- (void) setLiveUpdating:(BOOL)n;
- (void) setParentScrollView:(id)n;
- (NSView *) topNibView;
- (NSSize) topNibViewNativeSize;


@end
