//
//  VVQCComposition.h
//  VDMX
//
//  Created by bagheera on 9/8/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <Quartz/Quartz.h>
#import "NSMutDictAdditions.h"




/*

NOTES ON THE LAYOUT OF QUARTZ COMPOSER DOCUMENTS:

- a QC file at its top level is a dictionary
- the "inputParameters" key of the top-level dictionary points to another dictionary
	- each key of "inputParameters" corresponds to a top-level published input
	- the item corresponding to each key is the default value of the input port. this
		does not describe the full splitter settings (no max/min/etc.), just the default value!
	- boolean, number, and string inputs work exactly as expected
	- colors are represented by a dict, keys "red", "green", "blue", and "alpha"
	- "structure" inputs- which do not have a default value- are not listed here!
		this means that i can't use this dict to populate inputs!
	- the value of "index" inputs (menus) is represented as the selected index (no strings!)
- the "portAttributes" key of the top-level dictionary points to another dictionary
	- each key of the "portAttributes" dict represents a top-level published input or output
	- the keys of "portAttributes" are the names of the published inputs and output
	- every item in "portAttributes" is a dictionary.
		- the "name" key of this dict points to a string
		- the value of the "name" string is either "Input" (if it's an input), or "Output" (if it's an output)
- the "rootPatch" key of the top-level dictionary points to another dictionary
	- the "state" key of "rootPatch" points to another dictionary.
	- the "state" dictionary has 3 keys of interest:
		- the "publishedInputPorts" key points to an array
			- each item in "publishedInputPorts" is a dict
				- the "key" key in the dict points to a string, which is the name of the published input
				- the "node" key in the dict points to a string, which is the name of the node
		- the "publishedOutputPorts" key points to an array, and is just like "publishedInputPorts"
		- the "nodes" key points to an array
			- each item in "nodes" is a dict
				- the "key" key in the dict points to a string, which is the name of the node
				- the "class" key in the dict points to a string, which is the name of the class
				- the "state" key in the dict points to another dict, which describes the state of the input

*/




@interface VVQCComposition : NSObject {
	NSString				*compositionPath;
	NSMutableDictionary		*publishedInputsDict;
	NSMutableDictionary		*publishedOutputsDict;
	NSString				*category;
	NSString				*description;
	BOOL					hasLiveInput;
}

+ (id) compositionWithFile:(NSString *)p;
- (id) initWithFile:(NSString *)p;

- (NSDictionary *) findSplitterForPublishedInputNamed:(NSString *)n inStateDict:(NSDictionary *)d;
- (NSDictionary *) findSplitterForPublishedOutputNamed:(NSString *)n inStateDict:(NSDictionary *)d;

- (BOOL) findVideoInputInStateDict:(NSDictionary *)d;
- (void) cleanUpStateDict:(NSMutableDictionary *)d;
//- (id) valueForInputKey:(NSString *)k;

- (NSDictionary *) publishedInputsDict;
- (NSDictionary *) publishedOutputsDict;

- (NSString *) compositionPath;
- (NSArray *) inputKeys;
- (NSString *) category;
- (NSString *) description;
- (BOOL) hasLiveInput;

@end
