//
//  VVWindowContentView.h
//  VDMX
//
//  Created by bagheera on 5/10/06.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DataSourceController.h"
#import "VVWindow.h"




@interface VVWindowContentView : NSView <VVSnapshotProtocol> {
	DataSourceController	*dataSourceController;
	id						buttonManager;
	id						sliderManager;
	id						menuController;
	BOOL					selected;
	NSRect					actualLocation;		//	the actual size of the object; changes with collapse/expand
	NSSize					expandedSize;		//	the size when expanded
	NSSize					minSize;			//	the minimum size; set on awakeFromNib:
	BOOL					collapsed;			//	whether or not i'm collapsed
	BOOL					allowResize;
	BOOL					resizePreserveAspect;	//	whether or not i preserve my aspect ratio on resize
	float					localAspectRatio;		//	width / height
	NSString				*objectName;
	NSDictionary			*titleFontAttribDict;
	
	BOOL					reRender;		//	if YES, then the paths get re-rendered in drawRect:
	BOOL					locked;		//	whether or not the window is locked
	BOOL					fullyLocked;	//	whether or not the window is fully locked (can't collapse/expand)
	NSBezierPath			*framePath;	//	the stroked/filled path around everything
	NSBezierPath			*titlePath;	//	the path of the bg of the title bar
	NSRect					collapseToggleRect;	//	the bounding rect of the toggle that collapses the box
	NSRect					resizeHandleRect;	//	the bounding rect of the resize handle
	
	NSBezierPath			*collapsedPath;
	NSBezierPath			*expandedPath;
	NSBezierPath			*resizePath;
	
	int						dragAction;	//	0=drag, 1=collapse/expand, 2=resize
	NSPoint					superClickLoc;	//	location in superview's coords of the mousedown, so i can calculate delta
	NSRect					clickFrame;	//	my frame on mousedown
	
	BOOL					affectedByChange;
	
	BOOL					cmdClick;		//	if i cmd-click on a window, it moves *just* the window- it doesn't drag other windows, too
	BOOL					optClick;		//	if i opt-click a window, it moves *all* adjacent windows
	
	NSImage					*bgImage;
	NSRect					bgImageRect;
}

- (void) moveWindowToDefaultLocation;

- (NSRect) convertBLScreenToTLScreen:(NSRect)c;
- (NSRect) convertTLScreenToBLScreen:(NSRect)c;

- (void) expand;
- (void) collapse;
- (void) moveWindowToHere:(NSRect)n;

//	notifications
- (void) windowLockedNotification:(NSNotification *)note;
- (void) dragSetupNotification:(NSNotification *)note;
- (void) calculateAffectedAreaNotification:(NSNotification *)note;
- (void) objectMoveNotification:(NSNotification *)note;
- (void) moveAffectedObjectsNotification:(NSNotification *)note;
- (void) windowSelectNotification:(NSNotification *)note;
- (void) windowSizePosReportNotification:(NSNotification *)note;
- (void) windowRedrawNotification:(NSNotification *) note;

//	internal
- (void) renderPaths;
- (BOOL) wantsDefaultClipping;
- (void) setActualLocation:(NSRect)newLocation;
- (void) windowBecameKey;
- (void) setBgImage:(NSImage *)n;
- (NSColor *) backgroundColor;		//	these look like key-value methods, but they're not- at all
- (NSColor *) frameColor;
- (NSColor *) titleBarColor;
- (NSDictionary *) getRelevantColorDict;

//	key-value
@property (readonly) NSRect actualLocation;
@property (readonly) int dragAction;
- (void) setReRender:(BOOL)newVal;
@property (assign, readwrite) NSString *objectName;
@property (readonly) BOOL collapsed;
@property (assign, readwrite) NSSize minSize;
@property (assign, readwrite) DataSourceController *dataSourceController;
@property (assign, readwrite) id buttonManager;
@property (assign, readwrite) id sliderManager;
- (void) setAllowResize:(BOOL)n;
@property (readonly) BOOL cmdClick;
@property (assign, readwrite) id menuController;
- (void) setLocalAspectRatio:(float)n;
@property (readonly) BOOL locked;
@property (readonly) BOOL fullyLocked;

@end
